<?php

/*
Plugin Name: Ping.fm
Plugin URI: http://mattjacob.com/journal/pingfm-wordpress-plugin
Description: Works with Ping.fm's Custom URL feature to collect blog entries, microblog entries, and status updates and post them to your blog. Status updates are posted to the sidebar and displayed with a widget. Blog entries and microblog entries can have certain metadata applied to them automatically. You can set the author, category, tags, and post status (published or draft) for all incoming Ping.fm entries.
Version: 1.0.0
Author: Matt Jacob
Author URI: http://mattjacob.com
*/

function mj_PingFmInstall() {
    global $wpdb;

    $table = $wpdb->prefix . 'pingfm';

    if ($wpdb->get_var("SHOW TABLES LIKE '$table'") != $table) {
        $sql = "
            CREATE TABLE $table (
                `id` int(10) unsigned NOT NULL auto_increment,
                `status` text NOT NULL,
                `timestamp` timestamp NOT NULL,
                PRIMARY KEY  (`id`)
            );
        ";
        
        require_once ABSPATH . 'wp-admin/includes/upgrade.php';
        dbDelta($sql);

        $options = array(
            'wdg_title'      => 'Status Updates',
            'wdg_prefix'     => 'From Ping.fm:',
            'wdg_limit'      => 7,
            'wdg_item_color' => '#333333',
            'wdg_ts_color'   => '#999999',
            'def_status'     => 'publish',
            'def_author'     => 1,
            'def_category'   => wp_create_category('Ping.fm'),
            'def_tags'       => 'pingfm, social networking, custom url',
        );
        
        update_option('pingfm_options', $options);
        update_option('pingfm_token', mj_PingFmTokenGenerator());
    }
}

function mj_PingFmTokenGenerator() {
    return md5(uniqid(rand(), true));
}

function mj_PingFm($args) {
    global $wpdb;
    extract($args);
    $options = get_option('pingfm_options');
    
    echo $before_widget;
    echo $before_title . $options['wdg_title'] . $after_title;

    $table = $wpdb->prefix . 'pingfm';

    if ($wpdb->get_var("SHOW TABLES LIKE '$table'") != $table) {
        echo "<p>It looks like the required database table doesn't exist yet. "
            . "It's not really a big deal, but you'll have to go and create it "
            . "manually in order for status updates to work properly.</p>";
    }
    else {
        if ($wpdb->get_var("SELECT COUNT(id) FROM $table") > 0) {
            $sql = "
                SELECT status, UNIX_TIMESTAMP(timestamp) AS timestamp 
                FROM $table 
                ORDER BY timestamp DESC 
                LIMIT {$options['wdg_limit']} 
            ";
            $res = $wpdb->get_results($sql);

            echo "<ul>\n";

            foreach ($res as $r) {
                echo '<li>' . $options['wdg_prefix'] . ' ' . make_clickable(htmlspecialchars($r->status, ENT_QUOTES)) . '<br />';
                echo '<span class="updated-time">' . human_time_diff($r->timestamp) . ' ago</span></li>' . "\n";
            }

            echo "</ul>\n";    
        }
        else {
            echo "<p>It looks like the author of this blog hasn't posted any "
                . "<a href=\"http://ping.fm\">status updates</a> yet. You should "
                . "bother him/her with a fair amount of persistence until something "
                . "gets done.</p>";
        }
    }

    echo $after_widget;
}

function mj_ScrubFormInput($str) {
    return (isset($str)) ? trim(strip_tags(stripslashes($str))) : false;
}

function mj_DisplayDbValue($str) {
    return htmlspecialchars($str, ENT_QUOTES);
}

function mj_PingFmControl() {
    $options = get_option('pingfm_options');

    if ($_POST['pingfm_is_submitted']) {
        $options['wdg_title']      = mj_ScrubFormInput($_POST['pingfm_title']);
        $options['wdg_prefix']     = mj_ScrubFormInput($_POST['pingfm_prefix']);
        $options['wdg_limit']      = mj_ScrubFormInput($_POST['pingfm_limit']);
        $options['wdg_item_color'] = mj_ScrubFormInput($_POST['pingfm_item_color']);
        $options['wdg_ts_color']   = mj_ScrubFormInput($_POST['pingfm_ts_color']);
        
        if (empty($options['wdg_title'])) { 
            $options['wdg_title'] = 'Status Updates'; 
        }
        
        if (!preg_match('/^\d{1,2}$/', $options['wdg_limit'])) {
            $options['wdg_limit'] = '7'; 
        }

        if (!preg_match('/^#[A-Za-z0-9]{6}$/', $options['wdg_item_color'])) {
            $options['wdg_item_color'] = '#333333';
        }
        
        if (!preg_match('/^#[A-Za-z0-9]{6}$/', $options['wdg_ts_color'])) {
            $options['wdg_ts_color'] = '#999999';
        }
        
        update_option('pingfm_options', $options);
    }

    $title      = mj_DisplayDbValue($options['wdg_title']);
    $prefix     = mj_DisplayDbValue($options['wdg_prefix']);
    $limit      = mj_DisplayDbValue($options['wdg_limit']);
    $item_color = mj_DisplayDbValue($options['wdg_item_color']);
    $ts_color   = mj_DisplayDbValue($options['wdg_ts_color']);

    echo <<<EOF
<input type="hidden" id="pingfm_is_submitted" name="pingfm_is_submitted" value="1" />
<p>
  <label for="pingfm_title">Title: 
    <input class="widefat" id="pingfm_title" name="pingfm_title" type="text" value="$title" />
  </label>
</p>
<p>
  <label for="pingfm_prefix">Prefix updates with: 
    <input class="widefat" id="pingfm_prefix" name="pingfm_prefix" type="text" value="$prefix" />
  </label>
</p>
<p>
  <label for="pingfm_item_color">Status update text color: 
    <input class="widefat" id="pingfm_item_color" name="pingfm_item_color" type="text" value="$item_color" />
  </label><br />
  <small>Specify colors as 6 hexadecimal characters</small>
</p>
<p>
  <label for="pingfm_ts_color">Timestamp text color: 
    <input class="widefat" id="pingfm_ts_color" name="pingfm_ts_color" type="text" value="$ts_color" />
  </label><br />
  <small>Leave the color fields blank to reset to defaults</small> 
</p>
<p>
  <label for="pingfm_limit">Number of updates to show:
    <input class="widefat" id="pingfm_limit" name="pingfm_limit" type="text" value="$limit" />
  </label><br />
  <small>A value between 5 and 10 seems to work well</small>
</p>

EOF;
}

function mj_PingFmInit() {
    register_sidebar_widget('Ping.fm', 'mj_PingFm');
    register_widget_control('Ping.fm', 'mj_PingFmControl');
    add_action('admin_menu', 'mj_PingFmAddAdminLink');
    add_action('admin_head', 'mj_PingFmAdminStyle');
    
    if (is_active_widget('mj_PingFm')) {
        add_action('wp_head', 'mj_PingFmStyle');
    }
}

function mj_PingFmAddAdminLink() {
    add_submenu_page('plugins.php', 'Ping.fm Custom URL', 'Ping.fm Custom URL', 'manage_options', 'pingfm-config-opts', 'mj_PingFmAdminPage');
}

function mj_PingFmAdminPage() {
    global $wpdb;

    if (!empty($_POST['is_submitted'])) {
        echo '<div id="message" class="updated fade"><p>Your settings have been stored away deep within the catacombs of the database.</p></div>';

        $options['def_status']   = $_POST['default_status'];
        $options['def_author']   = $_POST['default_author'];
        $options['def_category'] = $_POST['default_category'];
        $options['def_tags']     = $_POST['default_tags'];

        update_option('pingfm_options', $options);

        if (!empty($_POST['regenerate_key'])) {
            update_option('pingfm_token', mj_PingFmTokenGenerator());
        }
    }
    
    $siteurl = get_option('siteurl');
    $path    = stristr(dirname(__FILE__), '/wp-content/plugins');
    $token   = get_option('pingfm_token');
    $options = get_option('pingfm_options');

    $endpoint = $siteurl . $path . '/ping.php?k=' . $token;
    
    $def_status_publish_check = ($options['def_status'] == 'publish') ? ' checked="checked"' : '';
    $def_status_draft_check   = ($options['def_status'] == 'draft')   ? ' checked="checked"' : '';

    $users = $wpdb->get_results("SELECT ID, user_login FROM $wpdb->users ORDER BY user_login");
    $authors = '<select name="default_author" id="default_author">' . "\n";
    
    foreach ($users as $u) {
        $selected   = ($u->ID == $options['def_author']) ? ' selected="selected"' : '';
        $first_name = $wpdb->get_var("SELECT meta_value FROM $wpdb->usermeta WHERE user_id = $u->ID AND meta_key = 'first_name'");
        $last_name  = $wpdb->get_var("SELECT meta_value FROM $wpdb->usermeta WHERE user_id = $u->ID AND meta_key = 'last_name'");

        if (trim($first_name) == '' || trim($last_name) == '') {
            $authors .= "<option value=\"{$u->ID}\"$selected>{$u->user_login}</option>\n";
        }
        else {
            $authors .= "<option value=\"{$u->ID}\"$selected>$first_name $last_name ({$u->user_login})</option>\n";
        }
    }
    
    $authors .= '</select>' . "\n";
    
    $categories = wp_dropdown_categories(array(
        'name'       => 'default_category',
        'selected'   => $options['def_category'],
        'orderby'    => 'name',
        'hide_empty' => 0,
        'echo'       => 0
    ));

    echo <<<EOF
<div class="wrap">
  
  <h2>Ping.fm Custom URL</h2>
  <div style="width:700px; margin:0 auto 20px;">
    
    <form action="" method="post">
    <input type="hidden" name="is_submitted" value="1" />

    <h3>Your Super Secret URL</h3>
    <p>Hi there!</p>
    <p>Let's get you up and running, alright?</p>
    <p>Log in to your Ping.fm account and head over to the 
      <a href="https://ping.fm/custom/">Custom URL Settings</a>. Copy and paste 
      the following URL into the 'Custom URL' box and then hit 'Submit'. Be sure 
      not to give out this URL to anyone, because if someone else were to get 
      ahold of it, they would be able to post status updates to your site. And
      they'd probably write embarrassing things about you, so that'd be no good
      at all.</p>
    <p><input type="text" value="$endpoint" class="code" style="width:100%;" /></p>
    <p>If that URL is too long for you, check out the end of the 
      <a href="http://wordpress.org/extend/plugins/pingfm-custom-url-status-updates/installation/">installation instructions</a>
      for a tip on how to use <code>mod_rewrite</code> to shorten it up.</p>
    
    <h3>A Fresh-Baked Plugin Key</h3>
    <p>So, you screwed up and now you need a new key, huh? No problem! Through
      the use of black magic, divination, witchcraft, and sorcery, we are now
      able to generate new keys upon request. There might also be some help 
      from a random number generator, an entropy generator, and the MD5 hashing
      algorithm, but that's not important.</p>
    <p>For reference, your current key is: <code>$token</code></p>
    <p>
      <label for="regenerate_key">
        <input type="checkbox" name="regenerate_key" id="regenerate_key" value="1" /> 
        Make me a new key, please! Kthx.
      </label>
    </p>

    <h3>Some Additional Settings</h3>
    <table class="form-table">
      <tr>
        <th>Default Status:</th>
        <td>
          <input type="radio" name="default_status" id="def_status_publish" value="publish"{$def_status_publish_check} /> 
          <label for="def_status_publish">Published</label>
          &nbsp;&nbsp;

          <input type="radio" name="default_status" id="def_status_draft" value="draft"{$def_status_draft_check} />
          <label for="def_status_draft">Draft</label>
        </td>
      </tr>
      <tr>
        <th><label for="default_author">Default Author:</label></th>
        <td>$authors</td>
      </tr>
      <tr>
        <th><label for="default_category">Default Category:</label></th>
        <td>$categories</td>
      </tr>
      <tr>
        <th><label for="default_tags">Default Tags:</label></th>
        <td><input type="text" name="default_tags" id="default_tags" size="40" value="{$options['def_tags']}" /> Separated by commas</td>
      </tr>
    </table>
    <p class="submit"><input type="submit" name="submit" value="Update Settings" /></p>
  
    </form>
  
  </div>
</div>

EOF;
}

function mj_PingFmAdminStyle() {
    echo <<<EOF
<style type="text/css">
input.narrowfat {
    border: 1px solid #cccccc;
    padding: 3px;
    text-align: center;
    width: 80px;
}
</style>

EOF;
}

function mj_PingFmStyle() {
    $options = get_option('pingfm_options');
    echo <<<EOF
<style type="text/css">
#sidebar #pingfm { }
#sidebar #pingfm .updated-time { color: {$options['wdg_ts_color']}; }
#sidebar #pingfm ul { margin-left: 0; }
#sidebar #pingfm ul li { color: {$options['wdg_item_color']}; margin: 0 0 10px; }
#sidebar #pingfm ul li:before { content: ""; }
</style>

EOF;
}

register_activation_hook(__FILE__, 'mj_PingFmInstall');
add_action('plugins_loaded', 'mj_PingFmInit');

?>
